/*	
	Copyright (c) 2007 J.D. Gehman, Melbourne Australia

    This file is part of BS-REDOR.
	It was introduced by John D. Gehman, Frances Separovic, Kun Lu, and
	Anil K. Mehta (2007) in J. Phys. Chem. B 111, pp7802-7811

    BS-REDOR is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    BS-REDOR is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "BoltzmannME.h"
#include "Paar.h"
#include "utils.h"
#include <gsl/gsl_siman.h>
using namespace std;

BoltzmannME::BoltzmannME( Paar &P ) {
//	P.seconds = time(NULL);
//	P.lfile << "Beginning Boltzmann ME: " << ctime(&P.seconds) << endl;
//	cout << "Beginning Boltzmann ME:\n";
}

int BoltzmannME::Lagrange_siman( Paar &P ) {
	P.seconds = time(NULL);
	P.lfile << "Beginning Boltzmann ME: " << ctime(&P.seconds) << endl;
	cout << "Beginning Boltzmann ME:\n";

	const gsl_rng_type * Tr;
	gsl_rng * r;
     
	gsl_rng_env_setup();
     
//	Tr = gsl_rng_default;
	Tr = gsl_rng_taus2;
	r = gsl_rng_alloc(Tr);
     
//	gsl_vector_set_all( P.lagrange,0.1);

	ofstream saout;	
	char saname[179];
	strcpy(saname,P.path);
	strcat(saname,P.outbase);
	strcat(saname,".sa");	
	saout.open(saname, ios::out);

	sprintf(P.str,"#%4s %7s %12s %10s %4s %4s %4s\n",
		"iter","evals","temp","chi2","#lss","#acc","$rej");
	saout << P.str;

	Paar x(P), new_x(P), best_x(P);  
	double E, new_E, best_E;
	int i, done;
	double T, u;
	int n_evals = 0, n_iter = 0, n_accepts, n_rejects, n_eless;

	E = P.calc_state_dataonly_nosim();
	best_E = E;
	double magicTemp;

	for (int ntry = 0; ntry < P.ntries; ntry++) {

		int iterT = int( gvg(P.iterT,ntry) );	
		double boltzmann = gvg(P.boltzmann,ntry);
		double maxstep = gvg(P.maxstep,ntry);
		double attentemp = gvg(P.attentemp,ntry);
		double mintemp = gvg(P.mintemp,ntry);
		double T = gvg(P.inittemp,ntry);
		if ( abs(T) < 1E-12 ) T = magicTemp;
		if ( abs(attentemp) < 12E-12 ) attentemp = gvg(P.attentemp,0);
		if ( abs(maxstep) < 12E-12 ) maxstep = gvg(P.maxstep,0);
		if ( abs(boltzmann) < 12E-12 ) boltzmann = gvg(P.boltzmann,0);
		if ( abs(mintemp) < 12E-12 ) mintemp = gvg(P.mintemp,0);
		if ( iterT == 0 ) iterT = int ( gvg(P.iterT,0) );

		sprintf(P.str,"\n%5d/%d B %8.2e S %8.2e Temp: %8.2e-%8.2e by %11.5e\n",
			iterT,ntry,boltzmann,maxstep,T,mintemp,attentemp);
		P.lfile << P.str;
		cout << P.str;
	
		done = 0;

		while (!done) {
			n_accepts = 0;
			n_rejects = 0;
			n_eless = 0;
    		for (i = 0; i < iterT; ++i) {
				new_x.copy_min(x);
				// take a step
				for (int lm=0; lm<P.nc; lm++) {
					u = gsl_rng_uniform(r);
					gsl_vector_set( new_x.lagrange, lm, u * 2.0 * maxstep - 
						maxstep + gvg( new_x.lagrange, lm) );
				}
				new_E = new_x.calc_state_dataonly_nosim();
				if ( !isfinite(new_E) ) {
//					cout << endl;
					for (int bs = 0; bs<62; bs++ ) cout << "\b";
					cout << "BANG. Chi2 is not finite at temp " << T ;
//						<< " avgdev: " << new_x.avgdev << endl;
//					for (int lg=0; lg<new_x.nc; lg++) 
//							cout << gvg( new_x.lagrange, lg ) << endl;
					++n_rejects;
					continue;
//					exit(0);
				}
				if(new_E <= best_E){
					best_x.copy_min(new_x);
	    	    	best_E=new_E;
					magicTemp = T;
				}
				// keep track of Ef() evaluations 
				++n_evals;
				//now take the crucial step: see if the new point is accepted
				//or not, as determined by the boltzman probability. If step is
				//is taken, write it to the .la file together with the 
				//Lagrange multiplier configuration.
				if (new_E < E) {
	    	    	// take a step 
					x.copy_min(new_x);
					E = new_E;
					++n_eless;
					
				} else if 
					(gsl_rng_uniform(r) < exp (-(new_E-E)/(boltzmann*T)) ) {
					// also take a step */
					x.copy_min(new_x);
					E = new_E;
					++n_accepts;
				} else {
					++n_rejects;
				}
    		}
	
			//print report
			char format[79];
			strcpy(format,"%5d %7d %12g %10.8f %4.2f %4.2f %4.2f ");
			sprintf(P.str,format,n_iter,n_evals,T,x.chi2, 
				100.0*double(n_eless)/double(iterT), 
				100.0*double(n_accepts)/double(iterT), 
				100.0*double(n_rejects)/double(iterT) );
			saout << P.str;
			for (int i=0; i<P.nc; i++) {
				sprintf(P.str,"%10.7le ", gvg(x.lagrange,i) );
				saout << P.str;
			}
			saout << endl;
	
			// Some updates to screen
			if ( double(n_iter)/5.0 - (n_iter/5) < 1E-6 ) {
				for (int bs = 0; bs<62; bs++ ) cout << "\b";
				printf("Temp %10.8f Current E %14.6g Best E %14.6g",T,E,best_E);
			}
	
	    	// apply the cooling schedule to the temperature 
			T /= attentemp;
			++n_iter;
			if (T < mintemp) done = 1;
		}
	}
	//at the end, copy the result onto the initial point
	P.copy_min(best_x);
	P.calc_state_dataonly_nosim();

	cout << endl;
	P.lfile << "CHI2   " <<	P.chi2 << endl;
	P.lfile << "AVGDEV " <<  P.avgdev << endl;
	P.lfile << "SCALING " << P.scale << endl;
	cout << "CHI2   " << 	P.chi2 << endl;
	cout << "AVGDEV " <<  P.avgdev << endl;
	cout << "SCALING " << P.scale << endl;

	saout.close();
	return 0;

}

int BoltzmannME::MC_error ( Paar &goodPaar  ) {
	goodPaar.seconds = time(NULL);
	goodPaar.lfile << "Beginning Boltzmann MC: " << ctime(&goodPaar.seconds) << endl;
	cout << "Beginning Boltzmann MC:\n";
	
	gsl_vector * distsum = gsl_vector_calloc( goodPaar.np );
	gsl_vector * distsumsq = gsl_vector_calloc( goodPaar.np );

	gsl_vector * blockavg = gsl_vector_calloc( goodPaar.np );
	gsl_vector * blockstdev = gsl_vector_calloc( goodPaar.np );

	gsl_vector * bl_avgsum = gsl_vector_calloc( goodPaar.np );
	gsl_vector * bl_avgsumsq = gsl_vector_calloc( goodPaar.np );
	gsl_vector * bl_stdevsum = gsl_vector_calloc( goodPaar.np );
	gsl_vector * bl_stdevsumsq = gsl_vector_calloc( goodPaar.np );

	gsl_vector * avgofavg = gsl_vector_calloc( goodPaar.np );
	gsl_vector * stdevofavg = gsl_vector_calloc( goodPaar.np );
	gsl_vector * avgofstdev = gsl_vector_calloc( goodPaar.np );
	gsl_vector * stdevofstdev = gsl_vector_calloc( goodPaar.np );

	gsl_vector * fracstdevofstdev = gsl_vector_calloc( goodPaar.np );

	gsl_vector * workspace = gsl_vector_calloc( goodPaar.np );

	const gsl_rng_type * T;
	gsl_rng * r;
     
	gsl_rng_env_setup();
     
	T = gsl_rng_default;
	r = gsl_rng_alloc(T);
    
	Paar E(goodPaar);

	int numblocks = 0;
	int numrejected = 0;
	double dataSSo, max_sofs, grand;
	while ( numblocks < E.maxmcblocks ) {
		++numblocks;

		gsl_vector_set_zero(distsum);
		gsl_vector_set_zero(distsumsq);
	
		for (int bl=0; bl<E.permcblock; bl++) {
			// Deep-copy Paar structure for error analysis
			E.copy_min(goodPaar);

			// add grandom error value to each data point with appropriate
			// sigma
			for ( int d=0; d<E.nc; d++ ) {
				dataSSo = gvg( E.redor, d );	
				grand = _grandom_sigma( r, gvg(E.error,d));
				gsl_vector_set( E.redor, d, dataSSo + grand);
//cout << "sigma: " << gvg(E.error,d) << " old: " << dataSSo << 
//	" grand: " << grand << " new: " << gvg(E.redor,d) << endl;
			}

			// run siman solver on noise-adjusted data, no printing
			BoltzmannME Fit(E);
			Fit.Lagrange_siman_MC(E);
//			E.calc_state();

//FIXME: rejection by rmsd selects for lower noise configurations. Should
//	rerun siman with slightly adjusted parameters so that distributions
//	are not biased
			goodPaar.lfile << numblocks << "." << bl << "Err CHI2 " 
				<< E.chi2 << " AVGDEV " <<  E.avgdev;
			cout << numblocks << "." << bl << "Err CHI2 " <<	
				E.chi2 << " AVGDEV " <<  E.avgdev;
		
			if ( E.chi2 > E.maxmcrmsd ) {
				numrejected += 1;
				goodPaar.lfile << " REJECTED" << endl;
				cout << " REJECTED" << endl;
				bl--;
				continue;
			} 
			cout << endl;
			goodPaar.lfile << endl;
		
			// update running sums and sum of squares for this block	
			gsl_vector_add(distsum,E.density);
			_gv_pow(E.density,2.0)	;
			gsl_vector_add(distsumsq,E.density);	
		}

		// average density value for the block above
		gsl_vector_memcpy( blockavg, distsum );
		gsl_vector_scale(blockavg,1.0/E.permcblock);
		//stdev of density values for the block above
		_gv_stdev(blockstdev,distsum,distsumsq,E.permcblock,workspace);

		// update running sums and sum of squares over all blocks thus far
		gsl_vector_add(bl_avgsum,blockavg);
		gsl_vector_add(bl_stdevsum,blockstdev);
		_gv_pow(blockavg,2.0);
		_gv_pow(blockstdev,2.0);
		gsl_vector_add(bl_avgsumsq,blockavg);
		gsl_vector_add(bl_stdevsumsq,blockstdev);

		if ( numblocks < 5 ) continue;

		// calculate stdev of stdev ...
		_gv_stdev(stdevofstdev,bl_stdevsum,bl_stdevsumsq,numblocks,workspace);

		// ... and test whether or not we're done
	gsl_vector_memcpy(	fracstdevofstdev, stdevofstdev);
	gsl_vector_div(fracstdevofstdev, avgofstdev);
		max_sofs = gsl_vector_max( stdevofstdev );	
		goodPaar.lfile << "BLOCK " << 
			numblocks << " max sigma err " << max_sofs << endl; 
		goodPaar.lfile << "rejection rate: " << 
			double(numrejected)/
				(double(numblocks)*double(E.permcblock)+double(numrejected)) 
				<< endl;
		cout << "BLOCK " << numblocks << " max sigma err " << max_sofs << endl; 
		cout << "rejection rate: " << 
			double(numrejected)/
				(double(numblocks)*double(E.permcblock)+double(numrejected)) 
				<< endl;
	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofavg,bl_avgsum);
	gsl_vector_scale(avgofavg,1.0/numblocks);

	// error in average
	_gv_stdev(stdevofavg,bl_avgsum,bl_avgsumsq,numblocks,workspace);
	
	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofstdev,bl_stdevsum);
	gsl_vector_scale(avgofstdev,1.0/numblocks);

	// should be same as above, but just in case... error in stdev
	_gv_stdev(stdevofstdev,bl_stdevsum,bl_stdevsumsq,numblocks,workspace);

	gsl_vector_memcpy( goodPaar.mcavg, avgofavg);
	gsl_vector_memcpy( goodPaar.mcavgerr, stdevofavg);
	gsl_vector_memcpy( goodPaar.mcstdev, avgofstdev);
	gsl_vector_memcpy( goodPaar.mcstdeverr, stdevofstdev);
		goodPaar.write_out();
		goodPaar.gnuplot_out();
	
		if ( max_sofs <= E.minstdev2 ) break;	
	}

	if (numblocks < goodPaar.maxmcblocks) {
		goodPaar.lfile 
			<< "Distributions at each discrete distance converged in " <<
			numblocks << " x " << goodPaar.permcblock << " mc points " << endl;
		cout << "Distributions at each discrete distance converged in " <<
			numblocks << " x " << goodPaar.permcblock << " mc points " << endl;
	} else {
		goodPaar.lfile 
			<< "Maximum monte carlo steps reached. Largest stdev is "<<
			max_sofs << endl;
		cout << "Maximum monte carlo steps reached. Largest stdev is "<<
			max_sofs << endl;
	}

	goodPaar.lfile << "number rejected: " << numrejected << " of total " 
		<< numblocks*goodPaar.permcblock << endl;
	cout << "number rejected: " << numrejected << " of total " 
		<< numblocks*goodPaar.permcblock << endl;

	// calculate stats for distribution and set in original Paar structure

	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofavg,bl_avgsum);
	gsl_vector_scale(avgofavg,1.0/numblocks);

	// error in average
	_gv_stdev(stdevofavg,bl_avgsum,bl_avgsumsq,numblocks,workspace);
	
	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofstdev,bl_stdevsum);
	gsl_vector_scale(avgofstdev,1.0/numblocks);

	// should be same as above, but just in case... error in stdev
	_gv_stdev(stdevofstdev,bl_stdevsum,bl_stdevsumsq,numblocks,workspace);

	gsl_vector_memcpy( goodPaar.mcavg, avgofavg);
	gsl_vector_memcpy( goodPaar.mcavgerr, stdevofavg);
	gsl_vector_memcpy( goodPaar.mcstdev, avgofstdev);
	gsl_vector_memcpy( goodPaar.mcstdeverr, stdevofstdev);

	gsl_vector_free(distsum);
	gsl_vector_free(distsumsq);
	gsl_vector_free(blockavg);
	gsl_vector_free(blockstdev);
	gsl_vector_free(bl_avgsum);
	gsl_vector_free(bl_avgsumsq);
	gsl_vector_free(bl_stdevsum);
	gsl_vector_free(bl_stdevsumsq);
	gsl_vector_free(avgofavg);
	gsl_vector_free(stdevofavg);
	gsl_vector_free(avgofstdev);
	gsl_vector_free(stdevofstdev);
	gsl_vector_free(workspace);
	return 0;

}

int BoltzmannME::Lagrange_siman_MC( Paar &P ) {
//FIXME: adapt this for use in MC: use MC simulated annealing parameters,
// and minimize report output.
	const gsl_rng_type * Tr;
	gsl_rng * r;
     
	gsl_rng_env_setup();
     
//	Tr = gsl_rng_default;
	Tr = gsl_rng_taus2;
	r = gsl_rng_alloc(Tr);
     
	Paar x(P), new_x(P), best_x(P);  
	double E, new_E, best_E;
	int i, done;
	double T, u;
	int n_evals = 1, n_iter = 0, n_accepts, n_rejects, n_eless;

	E = P.calc_state_dataonly_nosim();
	best_E = E;

	T = P.errinittemp;
	done = 0;

	while (!done) {
		n_accepts = 0;
		n_rejects = 0;
		n_eless = 0;
    	for (i = 0; i < P.erriterT; ++i) {
			new_x.copy_min(x);
			// take a step
			for (int lm=0; lm<P.nc; lm++) {
				u = gsl_rng_uniform(r);
				gsl_vector_set( new_x.lagrange, lm, u * 2.0 * P.errmaxstep - 
					P.errmaxstep + gvg( new_x.lagrange, lm) );
			}
			new_E = new_x.calc_state_dataonly_nosim();
			if ( !isfinite(new_E) ) {
				for (int bs = 0; bs<62; bs++ ) cout << "\b";
				cout << "BANG. MC Chi2 is not finite at temp " << T 
					<< " avgdev: " << new_x.avgdev;
				for (int lg=0; lg<new_x.nc; lg++) 
					cout << gvg( new_x.lagrange, lg ) << endl;
				exit(0);
			}
			if(new_E <= best_E){
				best_x.copy_min(new_x);
	        	best_E=new_E;
			}
			// keep track of Ef() evaluations 
			++n_evals;
			//now take the crucial step: see if the new point is accepted
			//or not, as determined by the boltzman probability 
			if (new_E < E) {
	        	// take a step 
				x.copy_min(new_x);
				E = new_E;
				++n_eless;
			} else if (gsl_rng_uniform(r) < 
				exp (-(new_E-E)/(P.errboltzmann*T)) ) {
				// also take a step */
				x.copy_min(new_x);
				E = new_E;
				++n_accepts;
			} else {
				++n_rejects;
			}
    	}

    	// apply the cooling schedule to the temperature 
		T /= P.errattentemp;
		++n_iter;
		if (T < P.errmintemp) done = 1;
	}

	//at the end, copy the result onto the initial point
	P.copy_min(best_x);
	P.calc_state_dataonly_nosim();

//	x.Paar::~Paar();
//	new_x.Paar::~Paar();
//	best_x.Paar::~Paar();

//	cout << endl;
	return 0;
}

/*
double MaximumEntropy_siman (void *);
int search_scale_me_siman( void *);
double wrapper_scale_me_siman(double , void *);
void Dstruct ( void * );
void *Cstruct ( void * );
void *Cstruct3 ( void * );
void Cpystruct (void *, void *);
void P1(void *xp);
void S1(const gsl_rng *, void *, double);
double M1(void *, void *);

double error_siman ( void *  );


double error_siman ( void * p ) {

	int np = ((struct Paar *) p)->np;
	int nc = ((struct Paar *) p)->nc;

	gsl_vector * distsum = gsl_vector_calloc( np );
	gsl_vector * distsumsq = gsl_vector_calloc( np );

	gsl_vector * blockavg = gsl_vector_calloc( np );
	gsl_vector * blockstdev = gsl_vector_calloc( np );

	gsl_vector * bl_avgsum = gsl_vector_calloc( np );
	gsl_vector * bl_avgsumsq = gsl_vector_calloc( np );
	gsl_vector * bl_stdevsum = gsl_vector_calloc( np );
	gsl_vector * bl_stdevsumsq = gsl_vector_calloc( np );

	gsl_vector * avgofavg = gsl_vector_calloc( np );
	gsl_vector * stdevofavg = gsl_vector_calloc( np );
	gsl_vector * avgofstdev = gsl_vector_calloc( np );
	gsl_vector * stdevofstdev = gsl_vector_calloc( np );

	gsl_vector * workspace = gsl_vector_calloc( np );

	const gsl_rng_type * T;
	gsl_rng * r;
     
	gsl_rng_env_setup();
     
	T = gsl_rng_default;
	r = gsl_rng_alloc(T);
    
	// User decides what's appropriate here, as we're probably pretty
	// close already. 
	gsl_siman_params_t params = {ntries, erriterT, errmaxstep, errboltzmann, 
		errinittemp, errattentemp, errmintemp}; 

	struct Paar E;
	E.cycles = gsl_vector_alloc(nc);
	E.Snot = gsl_vector_alloc(nc);
	E.S = gsl_vector_alloc(nc);
	E.redor = gsl_vector_alloc(nc);
	E.SoverSnot = gsl_vector_alloc(nc);
	E.avg = gsl_vector_alloc(nc);
	E.error = gsl_vector_alloc(nc);
	E.sqrerr = gsl_vector_alloc(nc);
	E.stdev = gsl_vector_alloc(nc);
	E.residualy = gsl_vector_alloc(nc);
	E.density = gsl_vector_alloc(np);
	E.dephase_by_cycles = gsl_matrix_alloc(nc,np);
	E.lagrange = gsl_vector_alloc(nc);
	E.mcavg = gsl_vector_calloc(np);
	E.mcavgerr = gsl_vector_calloc(np);
	E.mcstdev = gsl_vector_calloc(np);
	E.mcstdeverr = gsl_vector_calloc(np);
	E.adjust = gsl_vector_calloc(nc);

	int numblocks = 0;
	int numrejected = 0;
	double dataSSo, max_sofs;
	while ( numblocks < maxmcblocks ) {
		++numblocks;

		gsl_vector_set_zero(distsum);
		gsl_vector_set_zero(distsumsq);
	
		for (int bl=0; bl<permcblock; bl++) {
			// Deep-copy Paar structure for error analysis
			deep_copy_Paar(p, &E);

			// add grandom error value to each data point with appropriate
			// sigma
			for ( int d=0; d<E.nc; d++ ) {
				dataSSo = gvg( E.redor, d );	
				gsl_vector_set( E.redor, d, 
					dataSSo + grandom_sigma( r, gvg(E.error,d)));
	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofavg,bl_avgsum);
	gsl_vector_scale(avgofavg,1.0/numblocks);

	// error in average
	_gv_stdev(stdevofavg,bl_avgsum,bl_avgsumsq,numblocks,workspace);
	
	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofstdev,bl_stdevsum);
	gsl_vector_scale(avgofstdev,1.0/numblocks);

	// should be same as above, but just in case... error in stdev
	_gv_stdev(stdevofstdev,bl_stdevsum,bl_stdevsumsq,numblocks,workspace);

	gsl_vector_memcpy( goodPaar.mcavg, avgofavg);
	gsl_vector_memcpy( goodPaar.mcavgerr, stdevofavg);
	gsl_vector_memcpy( goodPaar.mcstdev, avgofstdev);
	gsl_vector_memcpy( goodPaar.mcstdeverr, stdevofstdev);
			}

			// run siman solver on noise-adjusted data, no printing
			gsl_siman_solve(r, p, &calc_state, S1, M1, NULL, 
				Cpystruct, Cstruct, Dstruct, 0, params);
			
			calc_state(&E);

//FIXME: rejection by rmsd selects for lower noise configurations. Should
//	rerun siman with slightly adjusted parameters so that distributions
//	are not biased
			lfile << "Err CHI2 " <<	E.rmsd << " Old RMSD " << E.oldrmsd 
				<< " AVGDEV " <<  E.avgdev;
			cout << "Err CHI2 " <<	E.rmsd << " Old RMSD " << E.oldrmsd 
				<< " AVGDEV " <<  E.avgdev;
		
			if ( E.rmsd > maxmcrmsd ) {
				numrejected += 1;
				lfile << " REJECTED" << endl;
				cout << " REJECTED" << endl;
				bl--;
				continue;
			} else {
				cout << endl;
				lfile << endl;
			}
		
			// update running sums and sum of squares for this block	
			gsl_vector_add(distsum,E.density);
			gv_pow(E.density,2.0)	;
//			gsl_vector_mul(E.density,E.density);	
			gsl_vector_add(distsumsq,E.density);	
		}

		// average density value for the block above
		gsl_vector_memcpy( blockavg, distsum );
		gsl_vector_scale(blockavg,1.0/permcblock);
		//stdev of density values for the block above
		gv_stdev(blockstdev,distsum,distsumsq,permcblock,workspace);

		// update running sums and sum of squares over all blocks thus far
		gsl_vector_add(bl_avgsum,blockavg);
		gsl_vector_add(bl_stdevsum,blockstdev);
		gv_pow(blockavg,2.0);
		gv_pow(blockstdev,2.0);
		gsl_vector_add(bl_avgsumsq,blockavg);
		gsl_vector_add(bl_stdevsumsq,blockstdev);

		if ( numblocks < 2 ) continue;

		// calculate stdev of stdev ...
		gv_stdev(stdevofstdev,bl_stdevsum,bl_stdevsumsq,numblocks,workspace);

		// ... and test whether or not we're done
		max_sofs = gsl_vector_max( stdevofstdev );	
		lfile << "BLOCK " << 
			numblocks << " max sigma err " << max_sofs << endl; 
		lfile << "rejection rate: " << 
			double(numrejected)/
				(double(numblocks)*double(permcblock)+double(numrejected)) 
				<< endl;
		cout << "BLOCK " << numblocks << " max sigma err " << max_sofs << endl; 
		cout << "rejection rate: " << 
			double(numrejected)/
				(double(numblocks)*double(permcblock)+double(numrejected)) 
				<< endl;
	
		if ( max_sofs <= minstdev2 ) break;	
	}

	if (numblocks < maxmcblocks) {
		lfile << "Distributions at each discrete distance converged in " <<
			numblocks << " x " << permcblock << " mc points " << endl;
		cout << "Distributions at each discrete distance converged in " <<
			numblocks << " x " << permcblock << " mc points " << endl;
	} else {
		lfile << "Maximum monte carlo steps reached. Largest stdev is "<<
			max_sofs << endl;
		cout << "Maximum monte carlo steps reached. Largest stdev is "<<
			max_sofs << endl;
	}

	lfile << "number rejected: " << numrejected << " of total " 
		<< numblocks*permcblock << endl;
	cout << "number rejected: " << numrejected << " of total " 
		<< numblocks*permcblock << endl;

	// calculate stats for distribution and set in original Paar structure

	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofavg,bl_avgsum);
	gsl_vector_scale(avgofavg,1.0/numblocks);

	// error in average
	gv_stdev(stdevofavg,bl_avgsum,bl_avgsumsq,numblocks,workspace);
	
	// average at each density point (should be same as data)
	gsl_vector_memcpy(avgofstdev,bl_stdevsum);
	gsl_vector_scale(avgofstdev,1.0/numblocks);

	// should be same as above, but just in case... error in stdev
	gv_stdev(stdevofstdev,bl_stdevsum,bl_stdevsumsq,numblocks,workspace);

	gsl_vector_memcpy( ((struct Paar *) p)->mcavg, avgofavg);
	gsl_vector_memcpy( ((struct Paar *) p)->mcavgerr, stdevofavg);
	gsl_vector_memcpy( ((struct Paar *) p)->mcstdev, avgofstdev);
	gsl_vector_memcpy( ((struct Paar *) p)->mcstdeverr, stdevofstdev);

	gsl_vector_free(distsum);
	gsl_vector_free(distsumsq);
	gsl_vector_free(blockavg);
	gsl_vector_free(blockstdev);
	gsl_vector_free(bl_avgsum);
	gsl_vector_free(bl_avgsumsq);
	gsl_vector_free(bl_stdevsum);
	gsl_vector_free(bl_stdevsumsq);
	gsl_vector_free(avgofavg);
	gsl_vector_free(stdevofavg);
	gsl_vector_free(avgofstdev);
	gsl_vector_free(stdevofstdev);
	gsl_vector_free(workspace);
	return 0;
}
*/
int	_gv_stdev( gsl_vector *stdev, const gsl_vector *sum, 
	const gsl_vector *sumsq, int n, gsl_vector *work) {

	gsl_vector_memcpy( work, sum);
	_gv_pow(work,2.0);
	gsl_vector_scale(work,-1.0);
	gsl_vector_memcpy( stdev, sumsq);
	gsl_vector_scale( stdev, double(n) );
	gsl_vector_add( stdev,work);
	gsl_vector_scale( stdev, 1.0/(double(n)*(double(n)-1.0) ) );
	_gv_pow( stdev, 0.5);
}	

int _gv_pow (gsl_vector * a, const double x) {
	const size_t N = a->size;
	double v;
	for (size_t i = 0; i < N; i++) {
		v = pow( gvg(a,i), x);	
		if ( !isfinite(v) &&  -gvg(a,i) < 1E-12 ) 
			gsl_vector_set( a,i,0 );
		else gsl_vector_set( a, i, v );
		if (!isfinite(gvg(a,i) ) ) {
			cout << "NAN" << endl; 
			exit(0);
		}
	}
	return GSL_SUCCESS;
}

double _grandom_sigma ( const gsl_rng * r, double sigma) {
	double sum = 0.0;
	for (int i=0; i<12; i++) sum += gsl_rng_uniform(r);
	return (sigma*(sum-6.0));
}

/*
double wrapper_scale_me_siman(double s, void *P) {
	((struct Paar *)P)->scale = s;
	MaximumEntropy_siman(P);
//	return( ((struct Paar *)P)->rmsd + abs( ((struct Paar *)P)->avgdev ) );
	return( ((struct Paar *)P)->rmsd );
//	return( abs( ((struct Paar *)P)->avgdev ) );
}

int search_scale_me_siman(void *P) {
	int max_iter = 100, status, iter = 0;

	gsl_function F;
	F.function = &wrapper_scale_me_siman;
	F.params = P;

	double fa,fb,fm;	
	double a = scaling1; 
	double b = scaling2;
	double m = (scaling1 + scaling2)/2.0;
//cout << "DEBUG 1\n";

//	const gsl_min_fminimizer_type *T;
//	T = gsl_min_fminimizer_brent;
	const gsl_min_fminimizer_type * T = gsl_min_fminimizer_brent;
//cout << "DEBUG 1a\n";
	gsl_min_fminimizer * s = gsl_min_fminimizer_alloc(T);
//	gsl_min_fminimizer *s;
//	s = gsl_min_fminimizer_alloc(T);
//cout << "DEBUG 1b\n";
	gsl_min_fminimizer_set(s, &F, m, a, b);
//cout << "DEBUG 1c\n";
	
	lfile << "minimizing (chi2 + avgdev) with " << gsl_min_fminimizer_name(s)
		<< endl;
	cout << "minimizing (chi2 + avgdev) with " << gsl_min_fminimizer_name(s)
		<< endl;

//cout << "DEBUG 2\n";
	sprintf(str,"%5s [%9s, %9s] %9s %9s\n", "iter", "lower", "upper",
		"min", "err(est)");
	lfile << str;
	cout << str;

	sprintf(str,"%5d [%.7f, %.7f] %.7f %.7f\n",iter,a,b,m,b-a);
	lfile << str;
	cout << str;
	
//cout << "DEBUG 3\n";
	do {
		iter++;
		status = gsl_min_fminimizer_iterate(s);

		m = gsl_min_fminimizer_x_minimum(s);
		a = gsl_min_fminimizer_x_lower(s);
		b = gsl_min_fminimizer_x_upper(s);
		fm = gsl_min_fminimizer_f_minimum(s);
		fa = gsl_min_fminimizer_f_lower(s);
		fb = gsl_min_fminimizer_f_upper(s);
//cout << "DEBUG 4\n";
		
		status = gsl_min_test_interval(a,b, 0.0002, 0);
	
		if (status == GSL_SUCCESS) {
			cout << "Converged:\n";
			lfile << "Converged:\n";
		}

		sprintf(str,"%5d [%.7f, %.7f] %.7f %.7f\n",iter,a,b,m,b-a);
		lfile << str;
		cout << str;
		sprintf(str,"%5d [%.7f, %.7f] %.7f\n\n",iter,fa,fb,fm);
		lfile << str;
		cout << str;

	} while (status == GSL_CONTINUE && iter < max_iter);	

	((struct Paar *)P)->scale = m;
	calc_state(P);
	lfile << "CHI2   " <<	((struct Paar *)P)->rmsd << endl;
	lfile << "OLD RMSD   " <<	((struct Paar *)P)->oldrmsd << endl;
	lfile << "AVGDEV " <<  ((struct Paar *)P)->avgdev << endl;
	lfile << "SCALING " << ((struct Paar *)P)->scale << endl;
	cout << "CHI2   " << 	((struct Paar *)P)->rmsd << endl;
	cout << "OLD RMSD   " << 	((struct Paar *)P)->oldrmsd << endl;
	cout << "AVGDEV " <<  ((struct Paar *)P)->avgdev << endl;
	cout << "SCALING " << ((struct Paar *)P)->scale << endl;

	return status;
	
}
*/
