/*	
	Copyright (c) 2007 J.D. Gehman, Melbourne Australia

    This file is part of BS-REDOR.
	It was introduced by John D. Gehman, Frances Separovic, Kun Lu, and
	Anil K. Mehta (2007) in J. Phys. Chem. B 111, pp7802-7811

    BS-REDOR is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    BS-REDOR is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
// VERSION 1.0 Multiple annealings
#include "Paar.h"
#include "utils.h"

Paar::Paar ( const char * inputs ) {
	// Open input parameters file 
    ifstream inparams;
    inparams.open(inputs, ios::in);
    if ( !inparams.is_open() ) {
        cout << "Can't open input params from " << inputs << endl;
        exit(0);
    }

	scaling2=0.0;
    /* Read inputs file. Convert the string variable to a char*, 
	   then use sscanf again to break-up the line into individual variables.*/
    getline(inparams,buf); sscanf(buf.c_str(), "MODE: %s", mode);
    getline(inparams,buf); sscanf(buf.c_str(), "LAMBDA_D: %s", lambdaDname);
    getline(inparams,buf); sscanf(buf.c_str(), "XYDATA: %s", SoSname);
    getline(inparams,buf); sscanf(buf.c_str(), "DISTDIST: %s", distname);
    getline(inparams,buf); sscanf(buf.c_str(), "PATH: %s", path);
    getline(inparams,buf); sscanf(buf.c_str(), "OUTPUT: %s", outbase);
    getline(inparams,buf); sscanf(buf.c_str(), "XN: %s", xN);
    getline(inparams,buf); sscanf(buf.c_str(), "YN: %s", yN);
	getline(inparams,buf); sscanf(buf.c_str(), "NEAR: %le", &near);
	getline(inparams,buf); sscanf(buf.c_str(), "FAR: %le", &far);
	getline(inparams,buf); sscanf(buf.c_str(), "GRAIN: %le", &grain);
    getline(inparams,buf); sscanf(buf.c_str(), "PER 2PI: %d", &pp2pi);
    getline(inparams,buf); sscanf(buf.c_str(), "MID PULSE: %le", &t1);
    getline(inparams,buf); sscanf(buf.c_str(), "MAS: %le", &MAS);
	getline(inparams,buf); 
		sscanf(buf.c_str(), "SCALING: %le %le", &scaling1, &scaling2);
    getline(inparams,buf); sscanf(buf.c_str(), "LAST POINT: %d", &lastpoint);
    getline(inparams,buf); sscanf(buf.c_str(), "NTRIES: %d", &ntries);
	
	iterT = gsl_vector_alloc(ntries);
	maxstep = gsl_vector_alloc(ntries);
	boltzmann = gsl_vector_alloc(ntries);
	inittemp = gsl_vector_alloc(ntries);
	attentemp = gsl_vector_alloc(ntries);
	mintemp = gsl_vector_alloc(ntries);

	int readptr, readptr_inc, tmpi, suc;
	double tmpd;

    //getline(inparams,buf); 
	//sscanf(buf.c_str(), "ITERS AT TEMP: %d", &iterT);
	readptr = readptr_inc = 0;
    getline(inparams,buf); 
	sscanf(buf.c_str()+readptr, "ITERS AT TEMP:%d%n", &tmpi,&readptr_inc);
	gsl_vector_set(iterT, 0, double(tmpi));
	for (int n=1; n<ntries; n++) {
		readptr += readptr_inc;
		suc = sscanf(buf.c_str()+readptr, "%d%n", &tmpi,&readptr_inc);
		if (suc != -1) gsl_vector_set(iterT, n, double(tmpi));
		else {
			for (int r=n; r<ntries; r++) 
				gsl_vector_set(iterT, r, gvg(iterT,0) );
			break;
		}
	}
    //getline(inparams,buf); 
	//sscanf(buf.c_str(), "MAX STEP SIZE: %le", &maxstep);
	readptr = readptr_inc = 0;
    getline(inparams,buf); 
	sscanf(buf.c_str()+readptr, "MAX STEP SIZE: %le%n", &tmpd,&readptr_inc);
	gsl_vector_set(maxstep, 0, tmpd);
	for (int n=1; n<ntries; n++) {
		readptr += readptr_inc;
		suc = sscanf(buf.c_str()+readptr, "%le%n", &tmpd,&readptr_inc);
		if (suc != -1) gsl_vector_set(maxstep, n, tmpd);
		else {
			for (int r=n; r<ntries; r++) 
				gsl_vector_set(maxstep, r, gvg(maxstep,0) );
			break;
		}
	}

    //getline(inparams,buf); 
	//sscanf(buf.c_str(), "BOLTZMANN: %le", &boltzmann);
	readptr = readptr_inc = 0;
    getline(inparams,buf); 
	sscanf(buf.c_str()+readptr, "BOLTZMANN: %le%n", &tmpd,&readptr_inc);
	gsl_vector_set(boltzmann, 0, tmpd);
	for (int n=1; n<ntries; n++) {
		readptr += readptr_inc;
		suc = sscanf(buf.c_str()+readptr, "%le%n", &tmpd,&readptr_inc);
		if (suc != -1) gsl_vector_set(boltzmann, n, tmpd);
		else {
			for (int r=n; r<ntries; r++) 
				gsl_vector_set(boltzmann, r, gvg(boltzmann,0) );
			break;
		}
	}

    //getline(inparams,buf); sscanf(buf.c_str(), "INIT TEMP: %le", &inittemp);
	readptr = readptr_inc = 0;
    getline(inparams,buf); 
	sscanf(buf.c_str()+readptr, "INIT TEMP: %le%n", &tmpd,&readptr_inc);
	gsl_vector_set(inittemp, 0, tmpd);
	for (int n=1; n<ntries; n++) {
		readptr += readptr_inc;
		suc = sscanf(buf.c_str()+readptr, "%le%n", &tmpd,&readptr_inc);
		if (suc != -1) gsl_vector_set(inittemp, n, tmpd);
		else {
			for (int r=n; r<ntries; r++) gsl_vector_set(inittemp, r, 0 );
			break;
		}
	}

    //getline(inparams,buf); 
	//sscanf(buf.c_str(), "ATTEN TEMP: %le", &attentemp);
	readptr = readptr_inc = 0;
    getline(inparams,buf); 
	sscanf(buf.c_str()+readptr, "ATTEN TEMP: %le%n", &tmpd,&readptr_inc);
	gsl_vector_set(attentemp, 0, tmpd);
	for (int n=1; n<ntries; n++) {
		readptr += readptr_inc;
		suc = sscanf(buf.c_str()+readptr, "%le%n", &tmpd,&readptr_inc);
		if (suc != -1) gsl_vector_set(attentemp, n, tmpd);
		else {
			for (int r=n; r<ntries; r++) 
				gsl_vector_set(attentemp, r, gvg(attentemp,0) );
			break;
		}
	}
	

    //getline(inparams,buf); 
	//sscanf(buf.c_str(), "MIN TEMP: %le", &mintemp);
	readptr = readptr_inc = 0;
    getline(inparams,buf); 
	sscanf(buf.c_str()+readptr, "MIN TEMP: %le%n", &tmpd,&readptr_inc);
	gsl_vector_set(mintemp, 0, tmpd);
	for (int n=1; n<ntries; n++) {
		readptr += readptr_inc;
		suc = sscanf(buf.c_str()+readptr, "%le%n", &tmpd,&readptr_inc);
		if (suc != -1) gsl_vector_set(mintemp, n, tmpd);
		else {
			for (int r=n; r<ntries; r++) 
				gsl_vector_set(mintemp, r, gvg(mintemp,0) );
			break;
		}
	}

    getline(inparams,buf); sscanf(buf.c_str(), "ERR ITERS AT TEMP: %d", 
		&erriterT);
    getline(inparams,buf); sscanf(buf.c_str(), "ERR MAX STEP SIZE: %le", 
		&errmaxstep);
    getline(inparams,buf); sscanf(buf.c_str(), "ERR BOLTZMANN: %le", 
		&errboltzmann);
    getline(inparams,buf); sscanf(buf.c_str(), "ERR INIT TEMP: %le", 
		&errinittemp);
    getline(inparams,buf); sscanf(buf.c_str(), "ERR ATTEN TEMP: %le", 
		&errattentemp);
    getline(inparams,buf); sscanf(buf.c_str(), "ERR MIN TEMP: %le", 
		&errmintemp);
    getline(inparams,buf); sscanf(buf.c_str(), "MIN STDEV2: %le", &minstdev2);
    getline(inparams,buf); sscanf(buf.c_str(), "PER MC BLOCK: %d", 
		&permcblock);
    getline(inparams,buf); sscanf(buf.c_str(), "MAX MC BLOCKS: %d", 
		&maxmcblocks);
    getline(inparams,buf); sscanf(buf.c_str(), "MAX MC CHI2: %le", &maxmcrmsd);
	inparams.close();

	if ( strcmp(xN,"C13") == 0 ) gX = gC;		
	else if ( strcmp(xN,"N15") == 0 ) gX = gN;		
	else if ( strcmp(xN,"D2") == 0 ) gX = gD;		
	else if ( strcmp(xN,"H2") == 0 ) gX = gD;		
	else if ( strcmp(xN,"F19") == 0 ) gX = gF;		
	else if ( strcmp(xN,"P31") == 0 ) gX = gP;		
	else if ( strcmp(xN,"H1") == 0 ) gX = gH;		
	else { 
		cout << "detect nucleus (X) not recognized" << endl;
		exit(1);
	}
	
	if ( strcmp(yN,"C13") == 0 ) gY = gC;		
	else if ( strcmp(yN,"N15") == 0 ) gY = gN;		
	else if ( strcmp(yN,"D2") == 0 ) gY = gD;		
	else if ( strcmp(yN,"H2") == 0 ) gY = gD;		
	else if ( strcmp(yN,"F19") == 0 ) gY = gF;		
	else if ( strcmp(yN,"P31") == 0 ) gY = gP;		
	else if ( strcmp(yN,"H1") == 0 ) gY = gH;		
	else { 
		cout << "dephasing nucleus (Y) not recognized" << endl;
		exit(1);
	}


	scale = scaling1;
	DDConstant = gX * gY * hbar * 1000.0 / pi2;

	// mkdir for all output files, restructure outbase so that everything
	// goes into this directory
	char command[179];
	strcpy(command,"mkdir ");
	strcat(command,path);
	strcat(command,outbase);
	system(command);

	strcat(path,outbase);
	strcat(path,"/");

	strcpy(command,"cp ");
	strcat(command,SoSname);
	strcat(command," ");
	strcat(command,path);
	system(command);

	strcpy(command,"cp ");
	strcat(command,inputs);
	strcat(command," ");
	strcat(command,path);
	system(command);

	// open log file
	char logfile[179];
	strcpy(logfile,path);
	strcat(logfile,outbase);
	strcat(logfile,".log");	
	lfile.open(logfile, ios::out);

//	char debugoutfile[179];
//	strcpy(debugoutfile,path);
//	strcat(debugoutfile,outbase);
//	strcat(debugoutfile,".bug");	
//	debugout.open(debugoutfile, ios::out);

	lfile << "MAXIMUM ENTROPY REDOR VERSION " << VERSION << endl;
	seconds = time(NULL);
	lfile << "Started: " << ctime( &seconds);
	lfile << endl << " * * * * * * * * INPUT PARAMETERS * * * * * * * * \n";
	lfile << "MODE: " << mode << endl;
	lfile << "LAMBDA_D: " << lambdaDname << endl;
	lfile << "XYDATA: " << SoSname << endl;
	lfile << "DISTDIST: " << distname << endl;
	lfile << "PATH: " << path << endl;
	lfile << "OUTPUT (basename): " << outbase << endl;
	lfile << "XN: " << xN << " (gamma = " << gX << ")\n";
	lfile << "XY: " << yN << " (gamma = " << gY << ")\n";
	lfile << "Calculated Dipolar Constant: " << DDConstant << endl;
	lfile << "Distances: " << near << " to " << far << " every " <<
		grain << " Angstroms\n";
	lfile << "PER 2PI: " << pp2pi << endl;
	lfile << "MID PULSE: " << t1 << endl;
	lfile << "MAS: " << MAS << endl;
	lfile << "LAST POINT: " << lastpoint << endl;
	lfile << "SCALING: " << scaling1 << " " << scaling2 << endl;
	lfile << "NTRIES: " << ntries << endl;
    lfile << "ITERS AT TEMP: "; 
	for (int n=0; n<ntries; n++) lfile << gvg(iterT,n) << " ";
	lfile << endl;
    lfile << "MAX STEP SIZE: ";
	for (int n=0; n<ntries; n++) lfile << gvg(maxstep,n) << " ";
	lfile << endl;
    lfile << "BOLTZMANN: ";
	for (int n=0; n<ntries; n++) lfile << gvg(boltzmann,n) << " ";
	lfile << endl;
    lfile << "INIT TEMP: ";
	for (int n=0; n<ntries; n++) lfile << gvg(inittemp,n) << " ";
	lfile << endl;
    lfile << "ATTEN TEMP: ";
	for (int n=0; n<ntries; n++) lfile << gvg(attentemp,n) << " ";
	lfile << endl;
    lfile << "MIN TEMP: ";
	for (int n=0; n<ntries; n++) lfile << gvg(mintemp,n) << " ";
	lfile << endl;
    lfile << "ERR ITERS AT TEMP: " << erriterT << endl;
    lfile << "ERR MAX STEP SIZE: " << errmaxstep << endl;
    lfile << "ERR BOLTZMANN: " << errboltzmann << endl;
    lfile << "ERR INIT TEMP: " << errinittemp << endl;
    lfile << "ERR ATTEN TEMP: " << errattentemp << endl;
    lfile << "ERR MIN TEMP: " << errmintemp << endl;
    lfile << "MIN STDEV2: " << minstdev2 << endl;
    lfile << "PER MC BLOCK: " << permcblock << endl;
    lfile << "MAX MC BLOCKS: " << maxmcblocks << endl;
    lfile << "MAX MC CHI2: " << maxmcrmsd << endl;

	lfile << flush;

	// read through data file once to count data points (nc)
	nc = -1;
	ifstream SoSdata;
	SoSdata.open( SoSname, ios::in );
	if ( !SoSdata.is_open() )	{
		cout << "Can't open datafile\n";
		exit(1);
	}
	while ( !SoSdata.eof() ) {
		getline(SoSdata,buf);
		if ( iscomment( buf ) ) continue;	
		nc++;
	}

	// allocate all vectors with dimension nc
	cycles = gsl_vector_alloc(nc);
	Snot = gsl_vector_alloc(nc);
	S = gsl_vector_alloc(nc);
	redor = gsl_vector_alloc(nc);
	SoverSnot = gsl_vector_calloc(nc);
	avg = gsl_vector_alloc(nc);
	error = gsl_vector_alloc(nc);
	sqrerr = gsl_vector_alloc(nc);
	stdev = gsl_vector_alloc(nc);
	residualy = gsl_vector_alloc(nc);
	lagrange = gsl_vector_calloc(nc);

	// reset file-read pointers to the beginning of the data file
	SoSdata.clear();
	SoSdata.seekg(0);

	// and read through file to pick up that data
	int cyc;
	double S0,Si,R,err;
	char datalogformat[79];
	strcpy(datalogformat,"cycles: %3d So: %10.4le S: %10.4le S/So %10.4le ");
	strcat(datalogformat,"err %10.4le\n");

	lfile << "Data from " << SoSname << endl;
	
	for (int r=0; r<nc; r++) {
		getline(SoSdata,buf);
		if ( iscomment( buf ) ) {
			r--;
			continue;
		}
		err = 0.0;	
		sscanf(buf.c_str(), "%d %le %le %le %le", &cyc, &S0, &Si, &R, &err);
		if ( err < 1E-12 || err >= 1 ) err = 1.0;

		sprintf(str, datalogformat, cyc, S0, Si, Si/S0, err);
		lfile << str << flush; 

		// compute redor curve for data
		gsl_vector_set( cycles, r, double(cyc) );
		gsl_vector_set( Snot, r, S0 );
		gsl_vector_set( S, r, Si );
		gsl_vector_set( redor, r, Si/S0 );
		gsl_vector_set( error, r, err );
		gsl_vector_set( sqrerr, r, pow(err,0.5) );
	}
	SoSdata.close();

	// Calculate any adjustments; i.e. if any distances are to be forced
	// in the distribution, calculate this data for all even cycle values
	// up to the maximum. 
	totaladjustdensity = 0.0;
	// but the data for adjustments to be calculated is..

	if ( lastpoint == 0 ) adjnc = int( gvg( cycles, nc-1 ) ) / 2 + 1;
	else adjnc = lastpoint/2 + 1;
	
	// read through distdist file once to count discrete distance points (np)
	adjust = gsl_vector_calloc(adjnc);
	SmoothME = gsl_vector_calloc(adjnc);
	adjnp = -1;
	ifstream distdist;
	distdist.open( distname, ios::in );
	if ( distdist.is_open() )	{
		while ( !distdist.eof() ) {
			getline(distdist,buf);
			if ( iscomment( buf ) ) continue;	
			adjnp++;
		}
		if (adjnp > 0 ) {
			adjdists = gsl_vector_calloc(adjnp);
			adjdensity = gsl_vector_alloc(adjnp);
		
			// reset file-read pointers to the beginning of the data file
			distdist.clear();
			distdist.seekg(0);

			lfile << endl << "Prescribed distance distribution (for simulation "
				<< "or forced into BME reconstruction" << endl;
		
			// read distances and densities
			double dis,den;
			for (int r=0; r< adjnp; r++) {
				getline(distdist,buf);
				if ( iscomment( buf ) ) {
					r--;
					continue;
				}	
				sscanf(buf.c_str(), "%le %le", &dis, &den);
				gsl_vector_set( adjdists,r,dis);
				gsl_vector_set( adjdensity, r, den);
				sprintf(str,"%10.4le %10.4le\n",dis,den);
				lfile << str;
				totaladjustdensity += den;
			}

			dephase_by_2sm = gsl_matrix_alloc(adjnc,adjnp);
		}
		distdist.close(); 
	}

	// determine how many distance values (np) to be used in distribution
	np = int( ( far - near) / grain ) + 2;

	if ( np < nc ) { 
		cout << "Warning: points in distance distribution < data points. "
			<< "You'll be happier in the end if you lower the GRAIN\n";
		lfile << "Warning: points in distance distribution < data points. "
			<< "You'll be happier in the end if you lower the GRAIN\n";
	}
	
	distances = gsl_vector_calloc(np);
	dd = gsl_vector_calloc(np);
	density = gsl_vector_calloc(np);
	mcavg = gsl_vector_calloc(np);
	mcavgerr = gsl_vector_calloc(np);
	mcstdev = gsl_vector_calloc(np);
	mcstdeverr = gsl_vector_calloc(np);

	dephase_by_2 = gsl_matrix_alloc(adjnc,np);
	
	for (int x=0; x < np; x++) {
		double d = near + x*grain;
		gsl_vector_set( distances, x, d );
		gsl_vector_set( dd, x, DDConstant / pow(d,3.0) );
	}

	// If lambdaD file is provided, use this to fill the transform matrix
	double maxlambdaD = gvg(dd,0) * lastpoint / MAS;
	cout << "maximum lambdaD value in analysis set-up: " << maxlambdaD << endl;
	lfile << "maximum lambdaD value in analysis set-up: " << maxlambdaD << endl;
	ifstream lambdaDin;
	lambdaDin.open( lambdaDname, ios::in );
	if ( lambdaDin.is_open() )	{
		cout << "reading matrix values from file\n";
		unsigned long int Ld=0;
		gsl_vector *lambdaD;
		gsl_vector *SS0;
		double x,y,D,L;

		while ( !lambdaDin.eof() ) {
			getline(lambdaDin,buf);
			if ( iscomment( buf ) ) continue;	
			sscanf(buf.c_str(), "%le", &x);
			if (x <= maxlambdaD ) Ld++;
		}

//		Ld--;
		lambdaD = gsl_vector_calloc(Ld);
		SS0 = gsl_vector_calloc(Ld);
		cout << "Size of input universal dephasing vector: "
			<< SS0->size << endl;

		// reset file-read pointers to the beginning of the data file
		lambdaDin.clear();
		lambdaDin.seekg(0);
		for ( int Lin=0; Lin < Ld; Lin++ ) {
			getline(lambdaDin,buf);
			if ( iscomment( buf ) ) continue;	
			sscanf(buf.c_str(), "%le %le", &x, &y);
			gsl_vector_set(lambdaD,Lin,x);
			gsl_vector_set(SS0,Lin,y);
		}

		// Now fill transform matrices with values derived from LambdaD input
		char tmp[5];

		// for prescribed distance (adjustments) ...
		if ( adjnp > 0 ) {
			cout << "Filling transform matrix elements for forced distances." 
				<< endl << " Of " << adjnp - 1 << ": ";

			for (int x=0; x < adjnp; x++) {
				sprintf(tmp,"%5i",x);
				cout << tmp << flush;
				D = ( DDConstant / pow(gvg( adjdists,x),3.0) ); // in Hz
				for (int c=0; c<adjnc; c++) {
					L = D * double(c) / MAS;
					// find index of LambdaD value closest to L
					double olddiff = -1*L;
					double newdiff;
					for (int ll=0; ll<Ld; ll++) {
						newdiff = gvg(lambdaD,ll) - L;
						if ( newdiff < 1E-12 ) {
							gsl_matrix_set( dephase_by_2sm,c,x,gvg(SS0,ll) );
							break;
						} else if ( newdiff < 0 && ll != Ld-1 ) {
							olddiff = newdiff;
							continue;
						} else if ( newdiff > 0 && ll != 0) {
							// interpolate between bracketing values
							gsl_matrix_set( dephase_by_2sm,c,x,
								gvg(SS0,ll) - 
								(gvg(SS0,ll) - gvg(SS0,ll-1) )
									*newdiff/(newdiff+olddiff) );
							break;
						} else if ( ll == 0 ) {
							break;
						} else {
							cout << "input REDOR curve not long enough\n";
							exit(0);
						}
					}
				}
				cout << "\b\b\b\b\b";
			}
			cout << endl;
		}
		// ... and for the full transform matrix
		cout << "Filling transform matrix elements (distance,cycles). " 
			<< endl << " Of " << np - 1 << ": ";
		for (int x=0; x < np; x++) {
			sprintf(tmp,"%5i",x);
			cout << tmp << flush;
			D = gsl_vector_get( dd, x);
			for (int c=0; c<adjnc; c++) {
				L = D * double(c) / MAS;
				// find index of LambdaD value closest to L
				double olddiff = -1*L;
				double newdiff;
				for (int ll=0; ll<Ld; ll++) {
					newdiff = gvg(lambdaD,ll) - L;
					if ( abs(newdiff) < 1E-12 ) {
						gsl_matrix_set( dephase_by_2,c,x,gvg(SS0,ll) );
						break;
					} else if ( newdiff < 0 && ll != Ld-1 ) {
						olddiff = newdiff;
						continue;
					} else if ( newdiff > 0 && ll != 0) {
						// interpolate between bracketing values
						gsl_matrix_set( dephase_by_2,c,x,
							gvg(SS0,ll) - 
							(gvg(SS0,ll) - gvg(SS0,ll-1) )
								*newdiff/(newdiff+olddiff) );
						break;
					} else if ( ll == 0 ) {
						break;
					} else {
						cout << "input REDOR curve not long enough\n";
						exit(0);
					}
				}
			}
			cout << "\b\b\b\b\b";
		}
	} else {	

		double d, D, sum, C, beta, sinbeta, Csin2beta;
		double alpha, Dfn;
		double pk = pi2*t1*MAS/1E6;
		double incpi = pi2/double(pp2pi);


		if ( adjnp > 0 ) {
			// loop through each value of redor-cycles, 
			// compute dephasing and redor transform matrix

			lfile << endl << " * * * * * * * * RAW DATA * * * * * * * * \n";

			cout << "Filling transform matrix elements for data adjustment. " 
				<< endl << " Of " << adjnc-1 << ": ";
		
			for (int c=0; c<adjnc; c++) {
				sprintf(str,"%4i",c);
				cout << str << flush;

			// and now loop through each distance value, calculating a
			// distance-specific contribution to dephasing	
				for (int x=0; x < adjnp; x++) {
					// distance (in Angstroms)
					d = gvg( adjdists,x);
					// corresponding dipolar coupling (in Hz)
					D = DDConstant / pow(d,3.0);
	
					sum=0;
					C = 4.0*pow(2.0,0.5)*D*double(c)/MAS;	// Hz/Hz
					// integrate over beta
					for (int b=0; b<pp2pi/2; b++) {
						beta = double(b)*incpi;
						sinbeta = sin(beta);
						Csin2beta = C*sin(2.0*beta);
						// integrate over alpha
						for (int a=0; a<pp2pi; a++) {
							alpha = double(a)*incpi;
							Dfn = Csin2beta * sin( alpha + pk );
							sum += sinbeta * cos( Dfn );
						}
					}
					sum *= 3.141593/(pow(double(pp2pi),2));
					gsl_matrix_set( dephase_by_2sm,c,x,sum);
				}
				cout << "\b\b\b\b";
			}
		}


		if ( strcmp( mode, "sim" ) ) {
			cout << "Filling transform matrix elements. " << endl 
				<< " Of " << adjnc - 1 << ": ";

			char tmp[5];
			for (int c=0; c<adjnc; c++) {
				sprintf(tmp,"%4i",c);
				cout << tmp << flush;
//				cout << c+1 << "." << flush;

				// loop through each distance value, calculating a
				// distance-specific contribution to dephasing	
				for (int x=0; x < np; x++) {
					// distance (in Angstroms)
					d = gvg( distances,x);
					// corresponding dipolar coupling (in Hz)
					D = gsl_vector_get( dd, x);
			
					sum=0.0;
					C = 4.0*pow(2.0,0.5)*D*double(c)/MAS;	// Hz/Hz
					// integrate over beta
					for (int b=0; b<pp2pi/2; b++) {
						beta = double(b)*incpi;
						sinbeta = sin(beta);
						Csin2beta = C*sin(2.0*beta);
						// integrate over alpha
						for (int a=0; a<pp2pi; a++) {
							alpha = double(a)*incpi;
							Dfn = Csin2beta * sin( alpha + pk );
							sum += sinbeta * cos( Dfn );
						}
						
					}
					sum *= 3.141593/(pow(double(pp2pi),2));
					gsl_matrix_set( dephase_by_2,c,x,sum);
				}
				cout << "\b\b\b\b";
			}

		}
	}			 
	// Cull the set of adjustments for the few which correspond to the data

	adjust_by_cycles = gsl_vector_calloc(nc);
	if ( adjnp > 0 ) {
		gsl_blas_dgemv(CblasNoTrans,1.0,dephase_by_2sm, 
			adjdensity, 0.0, adjust );
		cout << endl << "adjustment calculated: totaladjustdensity " 
			<< totaladjustdensity << endl;
		int c = 0;
		for (int c2=0; c2<nc; c2++) {
			for (int c1=c; c1<adjnc; c1++) {
				if ( int(gvg(cycles,c2)) == c1*2 ) {
					gsl_vector_set( adjust_by_cycles, c2, gvg(adjust,c1));
					c = c1;
				}
			}
		}
	}
	// create the reduce matrix where only elements corresponding
	// to data cycles exist
	dephase_by_cycles = gsl_matrix_alloc(nc,np);
	int c = 0;
	for (int c2=0; c2<nc; c2++) {
		for (int c1=c; c1<adjnc; c1++) {
			if ( int(gvg(cycles,c2)) == c1*2 ) {
				for (int x=0; x<np; x++) {
					gsl_matrix_set( dephase_by_cycles, c2,x, 
						gmg(dephase_by_2,c1,x) );
				}
				c = c1;
			}
		}
	}
	char matname[179];
	strcpy(matname,path);
	strcat(matname,outbase);
	strcat(matname,".mx");	
	ofstream matrix;
	matrix.open(matname, ios::out);
	for (int c=0; c<nc; c++ ) {
		for (int d=0; d<np; d++) {
			sprintf(str,"%10.6le ",
				gmg(dephase_by_cycles,c,d));
			matrix << str;
		}
		matrix << endl;
	}	
	matrix.close();
	
	strcat(matname,"T");	
	matrix.open(matname, ios::out);
	for (int d=0; d<np; d++) {
		for (int c=0; c<nc; c++ ) {
			sprintf(str,"%10.6le ",
				gmg(dephase_by_cycles,c,d));
			matrix << str;
		}
		matrix << endl;
	}	
	matrix.close();

	cout << endl;
	
}

Paar::Paar( const Paar &oldPaar ) {
	// New class object is implied
	np = oldPaar.np;
	nc = oldPaar.nc;
	adjnp = oldPaar.adjnp;
	adjnc = oldPaar.adjnc;
	ntries = oldPaar.ntries;
	
//	cycles = gsl_vector_alloc(nc);
//	Snot = gsl_vector_alloc(nc);
//	S = gsl_vector_alloc(nc);
	redor = gsl_vector_alloc(nc);
	SoverSnot = gsl_vector_alloc(nc);
//	avg = gsl_vector_alloc(nc);
//	error = gsl_vector_alloc(nc);
//	sqrerr = gsl_vector_alloc(nc);
//	stdev = gsl_vector_alloc(nc);
	residualy = gsl_vector_alloc(nc);
	density = gsl_vector_alloc(np);
//	distances = gsl_vector_alloc(np);
	lagrange = gsl_vector_alloc(nc);
//	mcavg = gsl_vector_calloc(np);
//	mcavgerr = gsl_vector_calloc(np);
//	mcstdev = gsl_vector_calloc(np);
//	mcstdeverr = gsl_vector_calloc(np);
//	adjust = gsl_vector_calloc(adjnc);
//	adjust_by_cycles = gsl_vector_calloc(nc);
//	adjdists = gsl_vector_calloc(adjnp);
//	SmoothME = gsl_vector_calloc(adjnc);
//	dephase_by_cycles = gsl_matrix_alloc(nc,np);
//	dephase_by_2 = gsl_matrix_alloc(adjnc,np);
//	iterT = gsl_vector_alloc(ntries);
//	inittemp = gsl_vector_alloc(ntries);
//	attentemp = gsl_vector_alloc(ntries);
//	boltzmann = gsl_vector_alloc(ntries);
//	maxstep = gsl_vector_alloc(ntries);
//	mintemp = gsl_vector_alloc(ntries);

	near = oldPaar.near;
	far = oldPaar.far;
	grain = oldPaar.grain;
	DDConstant = oldPaar.DDConstant;
	pp2pi = oldPaar.pp2pi;
	t1 = oldPaar.t1;
	MAS = oldPaar.MAS;
	chi2 = oldPaar.chi2;
	avgdev = oldPaar.avgdev;
	scale = oldPaar.scale;
	totaladjustdensity = oldPaar.totaladjustdensity;
	erriterT = oldPaar.erriterT;
	permcblock = oldPaar.permcblock;
	maxmcblocks = oldPaar.maxmcblocks;
	errmaxstep = oldPaar.errmaxstep;
	errboltzmann = oldPaar.errboltzmann;
	errinittemp = oldPaar.errinittemp;
	errattentemp = oldPaar.errattentemp;
	errmintemp = oldPaar.errmintemp;
	minstdev2 = oldPaar.minstdev2;
	maxmcrmsd = oldPaar.maxmcrmsd;

	cycles = oldPaar.cycles;
	Snot = oldPaar.Snot;
	S = oldPaar.S;
	avg = oldPaar.avg;
	error = oldPaar.error;
	sqrerr = oldPaar.sqrerr;
	stdev = oldPaar.stdev;
	distances = oldPaar.distances;
	adjust = oldPaar.adjust;
	adjust_by_cycles = oldPaar.adjust_by_cycles;
	SmoothME = oldPaar.SmoothME;
	dephase_by_cycles = oldPaar.dephase_by_cycles;
	dephase_by_2 = oldPaar.dephase_by_2;
	iterT = oldPaar.iterT;
	inittemp = oldPaar.inittemp;
	attentemp = oldPaar.attentemp;
	boltzmann = oldPaar.boltzmann;
	maxstep = oldPaar.maxstep;
	mintemp = oldPaar.mintemp;

//	gsl_vector_memcpy( cycles, oldPaar.cycles);
//	gsl_vector_memcpy( Snot, oldPaar.Snot);
//	gsl_vector_memcpy( S, oldPaar.S);
	gsl_vector_memcpy( redor, oldPaar.redor);
	gsl_vector_memcpy( SoverSnot, oldPaar.SoverSnot);
//	gsl_vector_memcpy( avg, oldPaar.avg);
//	gsl_vector_memcpy( error, oldPaar.error);
//	gsl_vector_memcpy( sqrerr, oldPaar.sqrerr);
//	gsl_vector_memcpy( stdev, oldPaar.stdev);
	gsl_vector_memcpy( residualy, oldPaar.residualy);
	gsl_vector_memcpy( density, oldPaar.density);
//	gsl_vector_memcpy( distances, oldPaar.distances);
	gsl_vector_memcpy( lagrange, oldPaar.lagrange);
//	gsl_vector_memcpy( adjust, oldPaar.adjust);
//	gsl_vector_memcpy( adjust_by_cycles, oldPaar.adjust_by_cycles);
//	gsl_vector_memcpy( adjdists, oldPaar.adjdists);
//	gsl_vector_memcpy( SmoothME, oldPaar.SmoothME);
//	gsl_matrix_memcpy( dephase_by_cycles, oldPaar.dephase_by_cycles);
//	gsl_matrix_memcpy( dephase_by_2, oldPaar.dephase_by_2);
//	gsl_vector_memcpy( iterT, oldPaar.iterT);
//	gsl_vector_memcpy( inittemp, oldPaar.inittemp);
//	gsl_vector_memcpy( attentemp, oldPaar.attentemp);
//	gsl_vector_memcpy( boltzmann, oldPaar.boltzmann);
//	gsl_vector_memcpy( maxstep, oldPaar.maxstep);
//	gsl_vector_memcpy( mintemp, oldPaar.mintemp);
}

int Paar::copy_min( const Paar &srcPaar ) {
	// Here it's important to deep-copy the lagrange multipliers.
	// The only other thing that needs to be copied is the chi2.
	// It's OK that everything else refers to the exact same thing
	// FIXME: beware. maybe the above not true
	chi2 = srcPaar.chi2;
	avgdev = srcPaar.avgdev;
	scale = srcPaar.scale;
	totaladjustdensity = srcPaar.totaladjustdensity;

	gsl_vector_memcpy( lagrange, srcPaar.lagrange );	
	gsl_vector_memcpy( SoverSnot, srcPaar.SoverSnot );	
	gsl_vector_memcpy( residualy, srcPaar.residualy );	
	gsl_vector_memcpy( redor, srcPaar.redor );
	gsl_vector_memcpy( density, srcPaar.density );
	
	return (1);
}

int Paar::copy_max( const Paar &srcPaar ) {
	
	chi2 = srcPaar.chi2;
    avgdev = srcPaar.avgdev;
    scale = srcPaar.scale;
    totaladjustdensity = srcPaar.totaladjustdensity;

    gsl_vector_memcpy( lagrange, srcPaar.lagrange );
    gsl_vector_memcpy( SoverSnot, srcPaar.SoverSnot );
    gsl_vector_memcpy( residualy, srcPaar.residualy );
    gsl_vector_memcpy( redor, srcPaar.redor );
    gsl_vector_memcpy( density, srcPaar.density );

	gsl_vector_memcpy( SmoothME, srcPaar.SmoothME );	
	gsl_vector_memcpy( cycles, srcPaar.cycles);
	gsl_vector_memcpy( Snot, srcPaar.Snot);
	gsl_vector_memcpy( S, srcPaar.S);
	gsl_vector_memcpy( avg, srcPaar.avg);
	gsl_vector_memcpy( error, srcPaar.error);
	gsl_vector_memcpy( sqrerr, srcPaar.sqrerr);
	gsl_vector_memcpy( stdev, srcPaar.stdev);
	gsl_vector_memcpy( distances, srcPaar.distances);
	gsl_vector_memcpy( adjust, srcPaar.adjust);
	gsl_vector_memcpy( adjust_by_cycles, srcPaar.adjust_by_cycles);
//	gsl_vector_memcpy( adjdists, srcPaar.adjdists);
	gsl_matrix_memcpy( dephase_by_cycles, srcPaar.dephase_by_cycles);
	gsl_matrix_memcpy( dephase_by_2, srcPaar.dephase_by_2);
	gsl_vector_memcpy( iterT, srcPaar.iterT);
	gsl_vector_memcpy( inittemp, srcPaar.inittemp);
	gsl_vector_memcpy( attentemp, srcPaar.attentemp);
	gsl_vector_memcpy( boltzmann, srcPaar.boltzmann);
	gsl_vector_memcpy( maxstep, srcPaar.maxstep);
	gsl_vector_memcpy( mintemp, srcPaar.mintemp);
}	

Paar::~Paar () {
    if ( lfile.is_open() ) {
		seconds = time(NULL);
		lfile << "time: " << ctime(&seconds) << endl;
		lfile << flush;
		lfile.close();
	}
	
	gsl_vector_free( lagrange );	
	gsl_vector_free( SoverSnot );	
	gsl_vector_free( residualy );	
	gsl_vector_free( redor );
	gsl_vector_free( density );
	
}

double Paar::calc_state() {
	// If we're just simulating, copy the data adjust vector over to SoverSnot
	if ( !strcmp( mode, "sim" ) ) {
		gsl_vector_memcpy(SoverSnot,adjust_by_cycles );
		gsl_vector_scale(SoverSnot,scale );
		gsl_vector_add_constant( SoverSnot, 1.0 - scale);
		// and for all rotor periods in between
		gsl_vector_memcpy(SmoothME,adjust );
		gsl_vector_scale(SmoothME,scale );
		gsl_vector_add_constant( SmoothME, 1.0 - scale);

		gsl_vector_memcpy(residualy, redor );
//		gsl_blas_daxpy(-1.0*scale,adjust_by_cycles, residualy );
//		gsl_vector_scale( residualy, 1.0/(1.0-totaladjustdensity) );
		gsl_blas_daxpy(-1.0,SoverSnot, residualy );
		gsl_vector_scale( residualy, -1.0);

	} else {

	// Use Lagrange multipliers to calculate the density distribution
	double Z;

	gsl_vector * sumvect = gsl_vector_alloc(np);
	gsl_vector * expvect = gsl_vector_alloc(np);
	gsl_vector * ones = gsl_vector_alloc(np);
	gsl_vector_set_all(ones,1.0);

	// Find partition coefficient (Z),
	gsl_blas_dgemv(CblasTrans,1.0,dephase_by_cycles, lagrange, 0.0, sumvect);
	for (int n=0; n<np; n++) gsl_vector_set(expvect,n, exp( gvg(sumvect,n)));
	gsl_blas_ddot(expvect,ones,&Z);
	
	// and density at each point in distribution
	gsl_vector_memcpy( density,expvect);
	gsl_vector_scale( density,1.0/Z);

	// as well as the corresponding REDOR curve for the data
	gsl_blas_dgemv(CblasNoTrans,1.0,dephase_by_cycles,density,0.0,SoverSnot );
	// as well as in between the data (every 2 rotor periods
	gsl_blas_dgemv(CblasNoTrans,1.0,dephase_by_2,density,0.0,SmoothME );

	gsl_vector_scale(SoverSnot,scale );
	gsl_vector_add_constant( SoverSnot, 1.0 - scale);
	gsl_vector_scale(SmoothME,scale );
	gsl_vector_add_constant( SmoothME, 1.0 - scale);

	// adj data to exclude the components specified in the adjustment file
	gsl_vector_memcpy(residualy, redor );
	gsl_blas_daxpy(-1.0*scale,adjust_by_cycles, residualy );
	gsl_vector_scale( residualy, 1.0/(1.0-totaladjustdensity) );
	gsl_blas_daxpy(-1.0,SoverSnot, residualy );
	gsl_vector_scale( residualy, -1.0);

	gsl_vector_free(sumvect);
	gsl_vector_free(expvect);
	gsl_vector_free(ones);
	}

	gsl_vector * onesnc = gsl_vector_alloc(nc);
	gsl_vector_set_all(onesnc,1.0);
	gsl_blas_ddot(residualy,onesnc,&avgdev);
	avgdev *= 1.0/nc;

	gsl_vector * tmp = gsl_vector_alloc(nc);
	gsl_vector_memcpy(tmp,residualy);
	gsl_vector_div(tmp,error);	
	
	chi2 = pow( gsl_blas_dnrm2( tmp ), 2.0);

	gsl_vector_free(onesnc);
	gsl_vector_free(tmp);

	return chi2;
}

double Paar::calc_state_dataonly_nosim() {
	// Use Lagrange multipliers to calculate the density distribution
	double Z;

	gsl_vector * sumvect = gsl_vector_alloc(np);
	gsl_vector * expvect = gsl_vector_alloc(np);
	gsl_vector * ones = gsl_vector_alloc(np);
	gsl_vector_set_all(ones,1.0);

	// Find partition coefficient (Z),
	gsl_blas_dgemv(CblasTrans,1.0,dephase_by_cycles, lagrange, 0.0, sumvect);
	for (int n=0; n<np; n++) gsl_vector_set(expvect,n, exp( gvg(sumvect,n)));
	gsl_blas_ddot(expvect,ones,&Z);

	// and density at each point in distribution 
	gsl_vector_memcpy( density,expvect);
	gsl_vector_scale( density,1.0/Z);

	// as well as the corresponding REDOR curve for the data
	gsl_blas_dgemv(CblasNoTrans,1.0,dephase_by_cycles,density,0.0,SoverSnot );

	gsl_vector_scale(SoverSnot,scale );
	gsl_vector_add_constant( SoverSnot, 1.0 - scale);

	// adj data to exclude the components specified in the adjustment file
	gsl_vector_memcpy(residualy, redor );

	gsl_blas_daxpy(-1.0*scale,adjust_by_cycles, residualy );
	gsl_vector_scale( residualy, 1.0/(1.0-totaladjustdensity) );
	gsl_blas_daxpy(-1.0,SoverSnot, residualy );
	gsl_vector_scale( residualy, -1.0);

	gsl_vector * onesnc = gsl_vector_alloc(nc);
	gsl_vector_set_all(onesnc,1.0);
	gsl_blas_ddot(residualy,onesnc,&avgdev);
	avgdev *= 1.0/nc;

	gsl_vector * tmp = gsl_vector_alloc(nc);
	gsl_vector_memcpy(tmp,residualy);
	gsl_vector_div(tmp,error);	
	
	chi2 = pow( gsl_blas_dnrm2( tmp ), 2.0);

	gsl_vector_free(sumvect);
	gsl_vector_free(expvect);
	gsl_vector_free(ones);
	gsl_vector_free(onesnc);
	gsl_vector_free(tmp);
	
	return chi2;
}

int Paar::write_out() {
	calc_state();
	ofstream outfile;
	char datname[179];
	strcpy(datname,path);
	strcat(datname,outbase);
	strcat(datname,".me");	
	outfile.open(datname, ios::out);

	char f1[179];
	char f2[179];

	strcpy(f1,"# %11s %13s %13s %13s %13s %13s %13s %13s %4s %13s %13s ");
	strcat(f1,"%13s %13s %13s %13s %13s %13s %13s %13s %4s %13s %13s");
	strcat(f1,"%13s %13s\n");
	sprintf(str,f1,"distance","dipolcoupling","density","integral",
		"MCaverage","MCaverage err","MCstdev","MCstdev err",
		"N_c","data_S0","data_S","data_S/S0","data_error",
		"fit_to_data","fit_adjust","residual","Lagrange_mult",
		"data_if_La_H-","data_if_La_H+","2_c","Smooth_fit",
		"Smooth_adjust","adj distances","adj density");
	outfile << str;

	strcpy(f1,"%13.8lf %13.8lf %13.8lf %13.8lf %13.8lf %13.8lf %13.8lf %13.8lf ");

	int i,c;
	double d, D,S0, Si, r, chk, res, adj, err;
	double dens, lg;	
	double integral = 0.0;
	double mca, mcae, mcs, mcse;

	int imax = adjnc;
	if ( np > imax) imax = np;
//	else imax = nc;

	for (int x=0; x < imax; x++) {
		if (x < np ) {
			d = gvg( distances, x);
			D = DDConstant / pow(d,3.0);
			dens = gvg( density,x);
			integral += dens;
			mca = gvg( mcavg,x);
			mcae = gvg( mcavgerr,x);
			mcs = gvg( mcstdev,x);
			mcse = gvg( mcstdeverr,x);
			sprintf(str, f1,d,D,dens,integral,mca,mcae,mcs,mcse);
			outfile << str;
		} else {
			sprintf(str, "%13s %13s %13s %13s %13s %13s %13s %13s ",
				"?","?","?","?","?","?","?","?");
			outfile << str;
		}
		if (x < nc) {
			c =  int(gvg( cycles, x) );
			S0 =  gvg( Snot, x);
			Si =  gvg( S, x);
			r =  gvg( redor, x);
			// to plot adj as if it was the only signal, you need to
			// 1.0 + scaling * (adj - 1.0)
			err =  gvg( error, x);
			if ( abs(err-1.0) < 1E-6 ) err = 0;
			adj = gvg(adjust_by_cycles,x);
			chk =  gvg( SoverSnot, x);
			res =  gvg( residualy, x);
			lg =  gvg( lagrange, x);
			if ( lg > 0.0 ) {
				strcpy(f2,"%4i %13.8lf %13.8lf %13.8lf %13.8lf %13.8lf ");
				strcat(f2,"%13.8lf %13.8lf %13.8lf %13s %13.8lf ");
				sprintf(str,f2, c,S0,Si,r,err,chk,adj,res,lg,"?",r);
			} else if (lg < 0.0) {
				strcpy(f2,"%4i %13.8lf %13.8lf %13.8lf %13.8lf %13.8lf ");
                strcat(f2,"%13.8lf %13.8lf %13.8lf %13.8lf %13s ");
                sprintf(str,f2, c,S0,Si,r,err,chk,adj,res,lg,r,"?");
			} else {
				strcpy(f2,"%4i %13.8lf %13.8lf %13.8lf %13.8lf %13.8lf ");
				strcat(f2,"%13.8lf %13.8lf %13.8lf %13s %13s ");
				sprintf(str,f2, c,S0,Si,r,err,chk,adj,res,lg,"?","?");
			}
			outfile << str;
		} else {
			strcpy(f2,"%4s %13s %13s %13s %13s %13s %13s ");
			strcat(f2,"%13s %13s %13s %13s ");
			sprintf(str,f2,"?","?","?","?","?","?","?","?","?","?","?");
           	outfile << str;
		}
		if (x < adjnc ) {
			sprintf(str,"%4i %13.8lf %13.8lf ",
				x*2, gvg(SmoothME,x), gvg(adjust, x) );
			outfile << str;
		} else {
			sprintf(str,"%4s %13s %13s ","?","?","?");
			outfile << str;
		}
		if (x < adjnp ) {
			sprintf(str,"%13.8lf %13.8lf",
				gvg(adjdists,x), gvg(adjdensity, x) );
			outfile << str;
		} else {
			sprintf(str,"%13s %13s","?","?","?");
			outfile << str;
		}
		outfile << endl;
	}
	outfile.close();
	return (1);
}

int Paar::gnuplot_out() {
	double minres = gsl_vector_min( residualy );
	for (int i=0; i<nc; i++ ) {
		double derr;
		if ( abs( gvg( error,i) - 1.0 ) < 1E-6 ) derr = gvg( redor,i);
		else derr =  gvg( redor,i) - gvg( error,i);
		if ( minres > derr ) minres = derr;
	}
	minres -= 0.0002;
	double maxdens = gsl_vector_max( density ) + 0.002;
	
	// left y-axis will run from -0.1 to 1. To get the zero axes to line up, 
	// maxdens must be 10 * minres.

	double unity2 = maxdens/10.0;
	if ( unity2 < abs(minres) ) unity2 = abs(minres);

	// write gnuplot script file to plot
	ofstream outfile;
	char datname[179];
	strcpy(datname,path);
	strcat(datname,outbase);
	strcat(datname,".gp");	
	outfile.open(datname, ios::out);
	outfile << "#set term postscript enhanced solid color" << endl
		<< "#set out '" << path << outbase << ".ps'" << endl
		<< "set style line 2 lw 1 lt 3 pt 6 ps 2" << endl
		<< "set style line 3 lw 1 lt 3 pt 8 ps 2" << endl
		<< "set style line 4 lw 1 lt 7 pt 0 ps 2" << endl
		<< "set xlabel 'Angstroms (red/green)'" << endl
		<< "set x2label 'Dephasing Time (ms) (blue/purple/aqua)'" << endl
		<< "set y2tics" << endl
		<< "set y2label 'density (red) residual (aqua)'" << endl
		<< "set ytics nomirror" << endl
		<< "set mxtics 10" << endl
		<< "set mx2tics 10" << endl
		<< "set mytics 10" << endl
		<< "set xrange [" << near << ":" << far << "]" << endl
		<< "set yrange [" << 1.0-scale-0.05 << ":1]" << endl
		<< "set x2range [0:*]" << endl
		<< "set x2tics" << endl
		<< "set xtics nomirror" << endl
		<< "set ylabel 'S/So (blue/purple) integral (green)'" << endl
		<< "set y2range [-" << unity2 << ":" << unity2*10.0 << "]" << endl;
	if ( !strcmp( mode, "fit" ) ) {
		outfile << "plot '" << path << outbase
		<< ".me' u 1:3 axes x1y2 w l lw 2 t "
		<< "'normalized density',\\" << endl
		<< "'' u 1:4 w l axes x1y1 t 'density integral',\\" << endl
		<< "'' u ($9*" << 1000.0/MAS << "):18:13 w "
		<< "yerrorbars ls 2 axes x2y1 t 'data H-',\\" << endl
		<< "'' u ($9*" << 1000.0/MAS << "):19:13 w "
		<< "yerrorbars ls 3 axes x2y1 t 'data H+',\\" << endl
		<< "'' u ($20*" << 1000.0/MAS << "):($21*" << 1.0-totaladjustdensity
		<< " +$22) w l axes x2y1 t 'ME reconstruction',\\" << endl
		<< "'' u ($9*" << 1000.0/MAS << "):16 w "
		<< "boxes ls 4 axes x2y2 t 'residual',\\" << endl
		<< 1.0-scale << " lt 0 notitle" << endl
		<< "#'' u 1:5:7 w yerrorbars axes x1y2 t 'Monte Carlo error'" << endl;
	} else if ( !strcmp( mode, "sim" ) ) {
		outfile << "set xrange [" << near << ":" << far << "]" << endl;
		outfile << "plot '" << path << outbase 
		<< ".me' u 23:24 axes x1y2 w l lw 2 t " << "'density',\\" << endl
        << "'' u ($9*" << 1000.0/MAS << "):12:13 w "
		<< "yerrorbars ls 2 axes x2y1 t 'data',\\" << endl
        << "'' u ($20*" << 1000.0/MAS 
		<< "):21 w l axes x2y1 t 'redor sim',\\" << endl
        << "'' u ($9*" << 1000.0/MAS 
		<< "):16 w boxes ls 4 axes x2y2 t 'residual',\\" << endl
        << 1.0-scale << " lt 0" << endl;
	} else if ( !strcmp( mode, "err" ) ) {
		outfile << "plot '" << path << outbase 
		<< ".me' u 1:3 axes x1y2 w l lw 2 t " 
		<< "'normalized density',\\" << endl
		<< "'' u 1:4 w l axes x1y1 t 'density integral',\\" << endl
        << "'' u ($9*" << 1000.0/MAS 
		<< "):18:13 w yerrorbars ls 2 axes x2y1 t 'data H-',\\" << endl
        << "'' u ($9*" << 1000.0/MAS 
		<< "):19:13 w yerrorbars ls 3 axes x2y1 t 'data H+',\\" << endl
		<< "'' u ($20*" << 1000.0/MAS << "):($21*" << 1.0-totaladjustdensity
		<< " +$22) w l axes x2y1 t 'ME reconstruction',\\" << endl
        << "'' u ($9*" << 1000.0/MAS 
		<< "):16 w boxes ls 4 axes x2y2 t 'residual',\\" << endl
        << 1.0-scale << " lt 0,\\" << endl
		<< "'' u 1:5:7 w yerrorbars axes x1y2 t 'Monte Carlo error'" << endl;
	} else {
		cout << "mode not recognized on output" << endl;
	}
	outfile.close();
	
	char command[179];
//	strcpy(command,"gnuplot ");
//	strcat(command,path);
//	strcat(command,datname);
//	system(command);

//	strcpy(command,"gv ");
//	strcat(command,outbase);
//	strcat(command,".ps &");
	if ( strcmp( mode, "err" ) ) {
		strcpy(command,"gnuplot -persist ");
		strcat(command,datname);
		system(command);
	}

	strcpy(datname,path);
	strcat(datname,outbase);
	strcat(datname,"-ps.gp");	
	outfile.open(datname, ios::out);
	outfile << "set term postscript eps enhanced solid color" << endl
		<< "set out '" << path << outbase << "-ps.eps'" << endl
		<< "set style line 2 lw 1 lt 3 pt 6 ps 2" << endl
		<< "set style line 3 lw 1 lt 3 pt 8 ps 2" << endl
		<< "set style line 4 lw 1 lt 7 pt 0 ps 2" << endl
		<< "set mxtics 10" << endl
		<< "set mx2tics 10" << endl
		<< "set mytics 10" << endl
		<< "set yrange [" << 1.0-scale-0.05 << ":1]" << endl
		<< "set size 1,1.5" << endl
		<< "unset key" << endl
		<< "#" << endl
		<< "set multiplot" << endl
		<< "# data/fit panel" << endl
		<< "set size 1,0.75" << endl
		<< "set origin 0,0.5" << endl
		<< "set xlabel 'dephasing time (ms)'" << endl
		<< "set ylabel 'S/S0'" << endl
		<< "set xrange [0:" << (1000.0/MAS)*(gvg(cycles,nc-1) + 8) << "]"
		<< endl << "set xtics mirror" << endl
		<< "plot '" << path << outbase 
		<< ".me'\\" << endl
		<< "     u ($9*" << 1000.0/MAS << "):18:13 w "
        << "yerrorbars ls 2 not,\\" << endl
		<< "  '' u ($9*" << 1000.0/MAS << "):19:13 w "
        << "yerrorbars ls 3 not,\\" << endl
		<< "  '' u ($20*" << 1000.0/MAS << "):($21*" << 1.0-totaladjustdensity
        << " +$22) w l ls 2 not,\\" << endl
		<< "  " << 1.0-scale << " lt 0 notitle" << endl
		<< "# distance distribution panel" << endl
		<< "set size 1.0,0.5" << endl
		<< "set origin 0.0,0.0" << endl
		<< "set xlabel 'distance (A)'" << endl
		<< "set ylabel 'intensity \\& integral" << endl
		<< "set xrange [*:*]" << endl
		<< "set yrange [0:1]" << endl
		<< "set y2range [0:*]" << endl
		<< "plot '" << path << outbase        
		<< ".me'\\" << endl 
		<< "     u 1:3 axes x1y2 w l ls 2 not,\\" << endl
		<< "  '' u 1:(1-$4) w l ls 2 not" << endl
		<< " #'' u 1:5:7 w yerrorbars not" << endl
		<< "# residuals panel" << endl
		<< "set size 1,0.25" << endl
		<< "set origin 0,1.25" << endl
		<< "set ylabel 'residual x 100'" << endl
		<< "unset xlabel" << endl
		<< "set mytics 2" << endl
		<< "set format x ''" << endl
		<< "set xrange [0:" << (1000.0/MAS)*(gvg(cycles,nc-1) + 8) << "]" 
		<< endl << "set yrange [*:*]" << endl
		<< "plot '" << path << outbase
		<< ".me'\\" << endl
		<< "     u ($9*" << 1000.0/MAS 
		<< "):(100*$16) w boxes ls 2 not" << endl;
	outfile.close();
	strcpy(command,"gnuplot ");
	strcat(command,datname);
	system(command);
	return 1;
}

int Paar::calc_dist_stats() {
	double r,D,I,r1,r2,r3,r4,D1,D2,D3,D4;
	I=r1=r2=r3=r4=D1=D2=D3=D4=0.0;
//	n = double(np);
	for (int i=0; i<np; i++) {
		r = gvg(density,i) * gvg(distances,i);
		D = gvg(density,i) * gvg(dd,i);
		r1 += r;
		D1 += D;
		I += gvg(density,i);
	}
	for (int i=0; i<np; i++) {
		r = gvg(distances,i);
		D = gvg(dd,i);
		r2 += pow( r - r1 , 2.0 ) * gvg(density,i);
		D2 += pow( D - D1 , 2.0 ) * gvg(density,i);
		r3 += pow( r - r1 , 3.0 ) * gvg(density,i);
		D3 += pow( D - D1 , 3.0 ) * gvg(density,i);
		r4 += pow( r - r1 , 4.0 ) * gvg(density,i);
		D4 += pow( D - D1 , 4.0 ) * gvg(density,i);
}
	r2=pow(r2,0.5);
	D2=pow(D2,0.5);
	r3 /= pow(r2,3.0);
	D3 /= pow(D2,3.0);
	r4 /= pow(r2,4.0);
	D4 /= pow(D2,4.0);
	cout << "total distribution density: " << I << endl;
	lfile << "total distribution density: " << I << endl;
	sprintf(str,"Distance moments: %13.8f %13.8f %13.8f %13.8f\n",r1,r2,r3,r4);
	lfile << str;
	cout << str;
	sprintf(str,"Frequency moments: %13.8f %13.8f %13.8f %13.8f\n",D1,D2,D3,D4);
	lfile << str;
	cout << str;

	return 1;
}

void Paar::set_lagrange(int i, double ell) {
	gsl_vector_set(lagrange,i,ell);
}

